#ifndef DATAFILE_H
#define DATAFILE_H

#include <string>
#include <vector>

#include "Common.h"
#include "array.h"
#include "pstring.h"

/* Datafile
 *
 * A support class for reading data from file, or writing data to file. */

class Datafile {
public:
    Datafile();
    Datafile(const char* filename);
    Datafile(FILE* fp);
    virtual ~Datafile();

    int NumColumns() const { return ncol; }
    int NumRows() const { return nrow; }
    const char* GetFilename() { return fname; }

    /***** Read mode *****/

    /* Read data from file */
    bool Read(const char* filename);
    bool Read(FILE* fp);

    /* Retrieve file metadata */
    const char* GetHeader() const { return header.c_str(); }
    const char* GetFooter() const { return footer.c_str(); }

    /* Retrieve data */
    const array& GetColumn(int j) const;
    array GetRow(int i) const;


    /***** Write mode *****/

    /* Write data to file */
    bool Write(const char* filename);
    bool Write() const;
    bool Write(FILE* fp) const;

    /* Set file metadata */
    void SetHeader(const char* s) { header = s; }
    void SetFooter(const char* s) { footer = s; }

    /* Set data */
    void AddColumn(const array& Y);
    void InsertColumn(const array& Y, int j);
    void AddRow(const array& X);
    void InsertRow(const array& X, int i);

protected:
    const char* fname;
    pstring header;
    pstring footer;

    std::vector<array> columns;
    int ncol;
    int nrow;
};

#endif // DATAFILE_H
