#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "GrowthFunction.h"
#include "LinearPS.h"


LinearPS::LinearPS(const Cosmology& C_, real z_) : C(C_) {
    z = z_;
    GrowthFunction D(C);

    /* Get transfer function */
    const array& k = C.ki;
    const array& T = C.Ti;

    /* Calculate P(k) */
    int N = k.size();
    array p(N);
    real A = pow2(C.delta_H) * pow2(D(z)/D(0));
    for(int i = 0; i < N; i++)
        p[i] = 2*M_PI*M_PI/pow3(k[i]) * A * pow(2997.925*k[i], 3+C.n) * pow2(T[i]);
    pk = CubicSpline(k, p);

    kf = k.front();
    pf = p.front();
    kb = k.back();
    pb = p.back();
}

real LinearPS::Evaluate(real k) const {
    if(k <= kf)
        return pf * pow(k/kf, C.n);
    else if(k >= kb) {
        real omh = C.Omega_m*C.h;
        real n = C.n;
        return pb * pow2(log(1.8*k/omh))*pow(k, n-4) / pow2(log(1.8*kb/omh))*pow(kb, 4-n);
    }
    else
        return pk(k);
}
