#ifndef SPT_H
#define SPT_H

#include "Common.h"


/**********************************************************************
 * SPT
 *
 * 1-loop standard perturbation theory.
 *********************************************************************/
class SPT {
public:
    SPT(const Cosmology& C, const PowerSpectrum& P_L, real epsrel = 1e-5);

    /* 1-loop power spectrum */
    real P(real k, int a = 1, int b = 1) const;

    /* 1-loop contributions to power spectrum */
    real P22(real k, int a = 1, int b = 1) const;
    real P13(real k, int a = 1, int b = 1) const;

    real P22_dd(real k) const;
    real P22_dt(real k) const;
    real P22_tt(real k) const;
    real P13_dd(real k) const;
    real P13_dt(real k) const;
    real P13_tt(real k) const;

    /* 1-loop propagator (normalized to 1) */
    real G(real k) const;

private:
    const Cosmology& C;
    const PowerSpectrum& P_L;
    real epsrel, epsabs;
};

#endif // SPT_H
